#<profile4xi profile = "10" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "get_command_line4xi.py">
#<abstract4xi abstract = "providing function get_command_line4xi"/>
#<refer4xi reference = "forking.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'get_command_line4xi' ]
#"
#/>
#

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:                 # winapi
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None
    
try:                 # 
    import itertools
    import weakref
    import atexit
    import msvcrt 
    from copyreg import dispatch_table
    from functools import partial
    import random
    import imp        
except:
    pass
try:
    from pickle import Pickler     
    from pickle import load, HIGHEST_PROTOCOL
except:
    pass     
#"
#/>


# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import forking
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from current_process4xi import get_identity_current_process4xi 
from logger4xi import Logger4Xi
from logger4xi import get_log_to_stderr4xi
from logger4xi import set_log_to_stderr4xi
from logger4xi import get_logger4xi
from logger4xi import set_logger4xi 
from is_forking4xi import is_forking4xi
from time4xi import time4xi
#"
#/>

# ---------------------------------------------------------------------------------------------
#<function4xi function = "get-command-line4xi" usage = "
#   forking.py
#       class Popen(object) in forking.py
#           def __init__(self, process_obj):
#              cmd = ' '.join('"%s"' % x for x in get_command_line())
#   popen4xi.py: <-- embedded function !
#       cmd2xi = get_command_line() + [rhandle2xi]
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# ---------------------------------------------------------------------------------------------
def get_command_line4xi():
    '''
    returns prefix of command line used for spawning a child process
    '''
    # ---------------------------------------------------------------------------------------------
    #
    # ---------------------------------------------------------------------------------------------    

    class2print = "get_command_line4xi"
    method2print = "get_command_line4xi"
    
    message2print = "{(Step 1/3a) started" + "; get-command-line4xi-started: " + str(get_command_line4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
        
    
    # ---------------------------------------------------------------------------------------------
    #
    # ---------------------------------------------------------------------------------------------     
    identity2xi = get_identity_current_process4xi()  #dragonized; value solved using function
    
    message2print = "{(Step 2/3a) set" + "; identity2xi-set-by-get-command-line4xi to: " + str(identity2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
        
    
    if identity2xi==() and is_forking4xi(sys.argv):     #dragonized;
        
        
        message2print = "{(Step 3/3z) hit" + "; RuntimeError-hit-by-get-command-line4xi: " + " *** " + "Attempt to start a new process before the current process has finished its bootstrapping phase"  + " *** " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        raise RuntimeError('''
        get_command_line4xi: Attempt to start a new process before the current process
        has finished its bootstrapping phase.
        This probably means that you are on Windows and you have
        forgotten to use the proper idiom in the main module:

            if __name__ == '__main__':
                freeze_support()
                ...

        The "freeze_support()" line can be omitted if the program
        is not going to be frozen to produce a Windows executable.''')

    if getattr(sys, 'frozen', False):
        
        message2print = "{(Step 3/3a) returning" + "; executable-returning-by-get-command-line4xi: " + str([sys.executable, '--multiprocessing-fork']) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        
        return [sys.executable, '--multiprocessing-fork']
    else:
        program2xi = 'from multiprocessing.forking import main; main()'
        
        message2print = "{(Step 3/3b) set" + "; program2xi-set-by-get-command-line4xi to: " + str(program2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        
        message2print = "{(Step 3/3c) returning" + "; python-exe-returning-by-get-command-line4xi: " + str([_python_exe, '-c', program2xi, '--multiprocessing-fork']) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        

        return [_python_exe, '-c', program2xi, '--multiprocessing-fork']    

#</function4xi> 
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>




# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of get_command_line4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

